"use strict";

exports.__esModule = true;
require("core-js/modules/es.error.cause.js");
function _classPrivateMethodInitSpec(e, a) { _checkPrivateRedeclaration(e, a), a.add(e); }
function _classPrivateFieldInitSpec(e, t, a) { _checkPrivateRedeclaration(e, t), t.set(e, a); }
function _checkPrivateRedeclaration(e, t) { if (t.has(e)) throw new TypeError("Cannot initialize the same private elements twice on an object"); }
function _classPrivateFieldGet(s, a) { return s.get(_assertClassBrand(s, a)); }
function _classPrivateFieldSet(s, a, r) { return s.set(_assertClassBrand(s, a), r), r; }
function _assertClassBrand(e, t, n) { if ("function" == typeof e ? e === t : e.has(t)) return arguments.length < 3 ? t : n; throw new TypeError("Private element is not present on this object"); }
var _rootElement = /*#__PURE__*/new WeakMap();
var _rootComputedStyle = /*#__PURE__*/new WeakMap();
var _rootDocument = /*#__PURE__*/new WeakMap();
var _isClassicTheme = /*#__PURE__*/new WeakMap();
var _cssVars = /*#__PURE__*/new WeakMap();
var _computedStyles = /*#__PURE__*/new WeakMap();
var _StylesHandler_brand = /*#__PURE__*/new WeakSet();
/**
 * Handles the theme-related style operations.
 */
class StylesHandler {
  /**
   * Initializes a new instance of the `StylesHandler` class.
   *
   * @param {object} domBindings - The DOM bindings for the instance.
   */
  constructor(domBindings) {
    var _classPrivateFieldGet2;
    /**
     * Retrieves and processes the computed styles for a `td` element.
     *
     * This method creates a temporary table structure, appends it to the root element,
     * retrieves the computed styles for the `td` element, and then removes the table
     * from the DOM. The computed styles are passed to the provided callback function.
     *
     * @param {Function} getterCallback - A callback function that processes the computed styles.
     *                                    It receives a `CSSStyleDeclaration` object as its argument.
     * @private
     */
    _classPrivateMethodInitSpec(this, _StylesHandler_brand);
    /**
     * The instance's root element.
     *
     * @type {HTMLElement}
     */
    _classPrivateFieldInitSpec(this, _rootElement, void 0);
    /**
     * The computed style of the root element.
     *
     * @type {CSSStyleDeclaration}
     * @private
     */
    _classPrivateFieldInitSpec(this, _rootComputedStyle, void 0);
    /**
     * The root document of the instance.
     *
     * @type {Document}
     * @private
     */
    _classPrivateFieldInitSpec(this, _rootDocument, void 0);
    /**
     * `true` if the classic theme is enabled, `false` otherwise.
     *
     * @type {boolean}
     */
    _classPrivateFieldInitSpec(this, _isClassicTheme, false);
    /**
     * An object to store CSS variable values.
     *
     * @type {object}
     * @private
     */
    _classPrivateFieldInitSpec(this, _cssVars, {});
    /**
     * Stores the computed styles for various elements.
     *
     * @type {object} - An object containing the computed styles if a nested structore of `element: { [element type]: {property: value} }`.
     * @private
     */
    _classPrivateFieldInitSpec(this, _computedStyles, {});
    _classPrivateFieldSet(_rootElement, this, domBindings.rootTable.parentElement);
    _classPrivateFieldSet(_rootComputedStyle, this, getComputedStyle(_classPrivateFieldGet(_rootElement, this)));
    _classPrivateFieldSet(_rootDocument, this, domBindings.rootDocument);
    if (((_classPrivateFieldGet2 = _classPrivateFieldGet(_rootComputedStyle, this)) === null || _classPrivateFieldGet2 === void 0 ? void 0 : _classPrivateFieldGet2.getPropertyValue('--ht-line-height')) === '') {
      _classPrivateFieldSet(_isClassicTheme, this, true);
    }

    // Some of the properties are not stored in the CSS variables, so we need to get them from the computed style.
    _assertClassBrand(_StylesHandler_brand, this, _getStylesForTD).call(this, computedStyle => {
      _classPrivateFieldSet(_computedStyles, this, {
        ..._classPrivateFieldGet(_computedStyles, this),
        ...{
          td: {
            'box-sizing': computedStyle['box-sizing'],
            'border-top-width': parseInt(computedStyle['border-top-width'], 10),
            'border-bottom-width': parseInt(computedStyle['border-bottom-width'], 10)
          }
        }
      });
    });
  }

  /**
   * Gets the value indicating whether the classic theme is enabled.
   *
   * @returns {boolean} `true` if the classic theme is enabled, `false` otherwise.
   */
  isClassicTheme() {
    return _classPrivateFieldGet(_isClassicTheme, this);
  }

  /**
   * Retrieves the value of a specified CSS variable.
   *
   * @param {string} variableName - The name of the CSS variable to retrieve.
   * @returns {number|string|undefined} The value of the specified CSS variable, or `undefined` if not found.
   */
  getCSSVariableValue(variableName) {
    if (_classPrivateFieldGet(_cssVars, this)[`--ht-${variableName}`]) {
      return _classPrivateFieldGet(_cssVars, this)[`--ht-${variableName}`];
    }
    const acquiredValue = _assertClassBrand(_StylesHandler_brand, this, _getParsedCSSValue).call(this, `--ht-${variableName}`);
    if (acquiredValue && acquiredValue !== '') {
      _classPrivateFieldSet(_cssVars, this, {
        ..._classPrivateFieldGet(_cssVars, this),
        ...{
          [variableName]: acquiredValue
        }
      });
      return acquiredValue;
    }
  }

  /**
   * Retrieves the computed style value for a specified CSS property of a `td` element.
   *
   * @param {string} cssProperty - The CSS property to retrieve the value for.
   * @returns {number|string|undefined} The value of the specified CSS property, or `undefined` if not found.
   */
  getStyleForTD(cssProperty) {
    var _classPrivateFieldGet3;
    const value = (_classPrivateFieldGet3 = _classPrivateFieldGet(_computedStyles, this)) === null || _classPrivateFieldGet3 === void 0 ? void 0 : _classPrivateFieldGet3.td[cssProperty];
    if (value !== undefined && value !== '') {
      return value;
    }
  }

  /**
   * Calculates the row height based on the current theme and CSS variables.
   *
   * @returns {number} The calculated row height.
   */
  getDefaultRowHeight() {
    if (_classPrivateFieldGet(_isClassicTheme, this)) {
      return 23;
    }
    return this.getStyleForTD('border-top-width') + this.getCSSVariableValue('cell-vertical-padding') * 2 + this.getCSSVariableValue('line-height') + this.getStyleForTD('border-bottom-width');
  }
}
exports.StylesHandler = StylesHandler;
function _getStylesForTD(getterCallback) {
  const table = _classPrivateFieldGet(_rootDocument, this).createElement('table');
  const tbody = _classPrivateFieldGet(_rootDocument, this).createElement('tbody');
  const tr = _classPrivateFieldGet(_rootDocument, this).createElement('tr');
  // This needs not to be the first row in order to get "regular" vaules.
  const tr2 = _classPrivateFieldGet(_rootDocument, this).createElement('tr');
  const td = _classPrivateFieldGet(_rootDocument, this).createElement('td');
  tr2.appendChild(td);
  tbody.appendChild(tr);
  tbody.appendChild(tr2);
  table.appendChild(tbody);
  _classPrivateFieldGet(_rootElement, this).appendChild(table);
  getterCallback(getComputedStyle(td));
  _classPrivateFieldGet(_rootElement, this).removeChild(table);
}
/**
 * Parses the value of a specified CSS property from the root element's computed style.
 *
 * @param {string} property - The CSS property to retrieve and parse.
 * @returns {number|string} The parsed value of the CSS property.
 */
function _getParsedCSSValue(property) {
  var _classPrivateFieldGet4;
  return parseInt((_classPrivateFieldGet4 = _classPrivateFieldGet(_rootComputedStyle, this)) === null || _classPrivateFieldGet4 === void 0 ? void 0 : _classPrivateFieldGet4.getPropertyValue(property), 10);
}